unit UtilList;

interface

uses
  SysUtils, Classes, Graphics, Controls, Forms, TypInfo, Dialogs, UtilEvents;

type
  TUtilList = class//(TUtilListBase)//TODO
    {Attributes}
    public Count: Integer;
    public IntegerKeys: array of Integer;
    public StringKeys: array of string;
    public BooleanList: array of Boolean;
    public ObjectList: array of TObject;
    public List: array of string;
    {Constructors}
    public constructor Create; overload;
    {Destructors}
    public destructor Destroy;
    {Methods}
    private procedure UpdateLength;
    private procedure Increment;
    private procedure Decrement;
    //Misc
    public function ToString: string;
    //BooleanList
    public procedure SetBooleanListByUtilList(true_list: TUtilList);
    public function GetBooleanTrueItems: TUtilList;
    public function GetBooleanFalseItems: TUtilList;
    //Add
    public function Add(integer_key: Integer; string_key: string; boolean_: Boolean; object_: TObject; value: string): Integer; overload;
    public function Add(string_key: string; value: string): Integer; overload;
    public function Add(integer_key: Integer; value: string): Integer; overload;
    public function Add(value: string): Integer; overload;
    public function Add(integer_key: Integer): Integer; overload;
    public function Add(object_: TObject): Integer; overload;
    public function Add(string_key: string; boolean_: Boolean; value: string): Integer; overload;
    public function Add(string_key: string; boolean_: Boolean; object_: TObject; value: string): Integer; overload;
    public function Add2(string_key: string; object_: TObject; value: string): Integer; overload;
    public function Add2(object_: TObject; value: string): Integer; overload;
    //Get
    public function GetCountByStringKey(string_key: string): Integer;
    public function GetCountByIntegerKey(integer_key: Integer): Integer;
    public function GetIntegerKeyByStringKey(string_key: string): Integer;
    public function GetIntegerKey(count_key: Integer): Integer;
    public function GetStringKeyByIntegerKey(integer_key: Integer): string;
    public function GetStringKey(count_key: Integer): string;
    public function GetBoolean(count_key: Integer): Boolean;
    public function GetBooleanByIntegerKey(integer_key: Integer): Boolean;
    public function GetBooleanByStringKey(string_key: string): Boolean;
    public function GetObject(count_key: Integer): TObject;
    public function GetObjectByIntegerKey(integer_key: Integer): TObject;
    public function GetObjectByStringKey(string_key: string): TObject;
    public function GetItem(count_key: Integer): string;
    public function GetItemByIntegerKey(integer_key: Integer): string;
    public function GetItemByStringKey(string_key: string): string;
    //Remove
    public function RemoveByStringKey(string_key: string): Boolean;
    public function RemoveByIntegerKey(integer_key: Integer): Boolean;
    public function Remove(count_key: Integer): Boolean;
    //Clear
    public procedure ClearList;
    {Events}
    public OnChange: TChangeEvent;

end;

implementation

{ TUtilList }

function TUtilList.Add(string_key, value: string): Integer;
begin
  Result := Add(-1, string_key, False, nil, value);
end;

function TUtilList.Add(integer_key: Integer; value: string): Integer;
begin
  Result := Add(integer_key, '', False, nil, value);
end;

function TUtilList.Add(object_: TObject): Integer;
begin
  Result := Add(-1, '', False, object_, '');
end;

function TUtilList.Add(string_key: string; boolean_: Boolean;
  value: string): Integer;
begin
  Result := Add(-1, string_key, boolean_, nil, value);
end;

function TUtilList.Add(string_key: string; boolean_: Boolean; object_: TObject;
  value: string): Integer;
begin
  Result := Add(-1, string_key, boolean_, object_, value);
end;

function TUtilList.Add2(string_key: string; object_: TObject;
  value: string): Integer;
begin
  Result := Add(-1, string_key, False, object_, value);
end;

function TUtilList.Add(value: string): Integer;
begin
  Result := Add(-1, '', False, nil, value);
end;

function TUtilList.Add(integer_key: Integer): Integer;
begin
  Result := Add(integer_key, '', False, nil, '');
end;

function TUtilList.Add2(object_: TObject; value: string): Integer;
begin
  Result := Add(-1, '', False, object_, value);
end;

function TUtilList.GetStringKeyByIntegerKey(integer_key: Integer): string;
var
  i: Integer;
begin
  i := GetCountByIntegerKey(integer_key);
  Result := '';
  if i > -1 then
    Result := StringKeys[i];
end;

function TUtilList.GetCountByIntegerKey(integer_key: Integer): Integer;
var
  i: Integer;
begin
  Result := -1;
  for i := 0 to Count - 1 do begin
    if IntegerKeys[i] = integer_key then
      Result := i;
  end;
end;

function TUtilList.GetCountByStringKey(string_key: string): Integer;
var
  i: Integer;
begin
  Result := -1;
  for i := 0 to Count - 1 do begin
    if StringKeys[i] = string_key then
      Result := i;
  end;
end;

function TUtilList.GetIntegerKeyByStringKey(string_key: string): Integer;
var
  key_number: Integer;
begin
  key_number := GetCountByStringKey(string_key);
  Result := -1;
  if key_number > -1 then
    Result := IntegerKeys[key_number];
end;

function TUtilList.GetBoolean(count_key: Integer): Boolean;
begin
  Result := BooleanList[count_key];
end;

function TUtilList.GetBooleanByIntegerKey(integer_key: Integer): Boolean;
begin
  Result := BooleanList[GetCountByIntegerKey(integer_key)];
end;

function TUtilList.GetBooleanByStringKey(string_key: string): Boolean;
begin
  Result := BooleanList[GetCountByStringKey(string_key)];
end;

function TUtilList.GetBooleanFalseItems: TUtilList;
var
  i: Integer;
begin
  Result := TUtilList.Create;
  for i := 0 to Count - 1 do begin
    if not GetBoolean(i) then begin
      Result.Add(GetIntegerKey(i), GetStringKey(i),
        GetBoolean(i), GetObject(i), GetItem(i));
    end;
  end;
end;

function TUtilList.GetBooleanTrueItems: TUtilList;
var
  i: Integer;
begin
  Result := TUtilList.Create;
  for i := 0 to Count - 1 do begin
    if GetBoolean(i) then begin
      Result.Add(GetIntegerKey(i), GetStringKey(i),
        GetBoolean(i), GetObject(i), GetItem(i));
    end;
  end;
end;

function TUtilList.GetIntegerKey(count_key: Integer): Integer;
begin
  if count_key > -1 then
    Result := IntegerKeys[count_key]
  else
    Result := 0;
end;

function TUtilList.GetItem(count_key: Integer): string;
begin
  Result := List[count_key];
end;

function TUtilList.GetItemByIntegerKey(integer_key: Integer): string;
begin
  Result := List[GetCountByIntegerKey(integer_key)];
end;

function TUtilList.GetItemByStringKey(string_key: string): string;
begin
  Result := List[GetCountByStringKey(string_key)];
end;

function TUtilList.GetObject(count_key: Integer): TObject;
begin
  Result := ObjectList[count_key];
end;

function TUtilList.GetObjectByIntegerKey(integer_key: Integer): TObject;
begin
  Result := GetObject(GetCountByIntegerKey(integer_key));
end;

function TUtilList.GetObjectByStringKey(string_key: string): TObject;
begin
  Result := GetObject(GetCountByStringKey(string_key));
end;

function TUtilList.GetStringKey(count_key: Integer): string;
begin
  Result := StringKeys[count_key];
end;

function TUtilList.Remove(count_key: Integer): Boolean;
var
  i: Integer;
begin
  Result := false;
  if count_key > -1 then begin
    IntegerKeys[count_key] := -1;
    StringKeys[count_key] := '';
    BooleanList[count_key] := False;
    if ObjectList[count_key] <> nil then begin
      ObjectList[count_key].Destroy;

//      ObjectList[count_key] := nil;
    end;
    List[count_key] := '';
    for i := count_key to Count - 1 do begin
      if count_key <= Count - 1 then begin
        IntegerKeys[i] := IntegerKeys[i+1];
        StringKeys[i] := StringKeys[i+1];
        BooleanList[i] := BooleanList[i+1];
        ObjectList[i] := ObjectList[i+1];
        List[i] := List[i+1];
      end else begin
        IntegerKeys[i] := -1;
        StringKeys[i] := '';
        BooleanList[i] := False;
        ObjectList[i].Free;
        List[i] := '';
      end;
    end;
    Decrement;
    Result := True;
  end;
end;

function TUtilList.RemoveByIntegerKey(integer_key: Integer): Boolean;
begin
  Result := Remove(GetCountByIntegerKey(integer_key));
end;

function TUtilList.RemoveByStringKey(string_key: string): Boolean;
begin
  Result := Remove(GetCountByStringKey(string_key));
end;

procedure TUtilList.SetBooleanListByUtilList(true_list: TUtilList);
var
  i, j: Integer;
begin
  for j := 0 to Count - 1 do begin
    BooleanList[j] := False;
  end;
  for i := 0 to true_list.Count - 1 do begin
    for j := 0 to Count - 1 do begin
      if ((true_list.GetIntegerKey(i) <> -1)  and (true_list.GetIntegerKey(i) = GetIntegerKey(j))) or
         ((true_list.GetStringKey(i)  <> '')  and (true_list.GetStringKey(i)  = GetStringKey(j))) or
         ((true_list.GetItem(i)       <> '')  and (true_list.GetItem(i)       = GetItem(j))) or
         ((true_list.GetObject(i)     <> nil) and (true_list.GetObject(i)     = GetObject(j))) then begin
        BooleanList[j] := True;
      end;
    end;
  end;
end;

function TUtilList.ToString: string;
var
  i: Integer;
begin
  Result := '';
  for i := 0 to Count - 1 do begin
    Result := Result + IntToStr(i) + '|';
    Result := Result + IntToStr(GetIntegerKey(i)) + '|';
    Result := Result + GetStringKey(i) + '|';
    if GetObject(i) <> nil then
      Result := Result + GetObject(i).ClassName + '|'
      else
      Result := Result + 'nil|';
    if GetBoolean(i) then
      Result := Result + 'True|'
      else
      Result := Result + 'False|';
    Result := Result + GetItem(i) + '|' + #13#10;
  end;
end;
procedure TUtilList.ClearList;
var
  i: Integer;
begin
  i := Count - 1;
  while i > -1 do begin
    Remove(i);
    i := i - 1;
  end;
  Count := 0;
  UpdateLength;
end;

constructor TUtilList.Create;
begin
  Count := 0;
  UpdateLength;
end;

function TUtilList.Add(integer_key: Integer; string_key: string;
  boolean_: Boolean; object_: TObject; value: string): Integer;
begin
  IntegerKeys[Count] := integer_key;
  StringKeys[Count] := string_key;
  BooleanList[Count] := boolean_;
  ObjectList[Count] := object_;
  List[Count] := value;
  Result := Count;
  Increment;
end;

procedure TUtilList.Decrement;
begin
  Count := Count - 1;
  UpdateLength;
end;

destructor TUtilList.Destroy;
begin
  ClearList;
  Finalize(IntegerKeys);
  Finalize(StringKeys);
  Finalize(BooleanList);
  Finalize(ObjectList);
  Finalize(List);
end;

procedure TUtilList.Increment;
begin
  Count := Count + 1;
  UpdateLength;
end;

procedure TUtilList.UpdateLength;
begin
  SetLength(IntegerKeys, Count+1);
  SetLength(StringKeys, Count+1);
  SetLength(BooleanList, Count+1);
  SetLength(ObjectList, Count+1);
  SetLength(List, Count+1);
  if Assigned(OnChange) then OnChange;
end;

end.
